package ar.com.zauber.commons.mom;

/**
 * An object for determining if objects of certain classes can be mapped into
 * maps
 */
public final class MappingPackage {
	private final String packageName;

	public MappingPackage(final String packageName) {
		this.packageName = packageName;
	}

	public boolean isMappeableArray(final Class<?> clazz) {
		return clazz.isArray() && isMappeable(clazz.getComponentType());
	}

	public boolean isMappeable(final Class<?> clazz) {
		return !clazz.isPrimitive() && !clazz.isArray() && packageContains(clazz);
	}

	protected boolean packageContains(final Class<?> clazz) {
		return clazz.getName().startsWith(packageName)
				|| (clazz.getSuperclass() != null && clazz.getSuperclass().getName().startsWith(packageName));
	}
}
